// Wait for the deviceready event before using any of Cordova's device APIs.
// See https://cordova.apache.org/docs/en/latest/cordova/events/events.html#deviceready
document.addEventListener('deviceready', onDeviceReady, false);

function onDeviceReady() {
    // Cordova is now initialized. Have fun!

    console.log('Running cordova-' + cordova.platformId + '@' + cordova.version);
    getCoordinates();
}

/* Google Maps */
var theMap;
var latitude = -1, prevLat; 
var longitude = -1, prevLon;
var the_distance = 0;
var press_start = false, select_pause = false;
var YOUR_KEY = "";

function getCoordinates() {
    navigator.geolocation.getCurrentPosition(onSuccess, onError);
    if(!select_pause) setTimeout(getCoordinates, 60000); // milliseconds
}

function onSuccess(pos) {
    // Update global variables
    prevLat = (latitude > 0) ? latitude : pos.coords.latitude;
    prevLon = (longitude > 0) ? longitude : pos.coords.longitude;;
    latitude = pos.coords.latitude;
    longitude = pos.coords.longitude;
    if(press_start) {
        the_distance = 0;
        press_start = false;
    } else {
        the_distance += geoDistance(prevLat, prevLon, latitude, longitude);
    }
    
    // insert
    sql="INSERT INTO route (timestamp, lat, lon, distance) VALUES (?, ?, ?, ?)";
    dbInst.execute(sql, [ getTimeStamp(), pos.coords.latitude, pos.coords.longitude, the_distance ], refreshAll);
    console.log('Latitude: '+ pos.coords.latitude + ' ' +
                'Longitude: '+ pos.coords.longitude);        
}

function onError(error) {
    alert('code: '    + error.code    + '\n' +
          'message: ' + error.message + '\n');
}

/* Google Maps */
function gMap() {
    $.getScript("https://maps.googleapis.com/maps/api/js?key="+YOUR_KEY+"&callback=gMapCallback");     
}

function gMapCallback() {
    center = new google.maps.LatLng(latitude, longitude);
    config = {center: center, zoom: 10,};
    theMap = new google.maps.Map(document.getElementById("googleMap"), config);
    console.log( "Map Created" );
    refreshAll();
}

/* WEBSQL */
function InitDB() { 
    sql="CREATE TABLE IF NOT EXISTS route (timestamp INTEGER, lat REAL, lon REAL, distance REAL)";
    dbInst.execute(sql);
}

function refreshAll() {
    sql="SELECT * FROM route";
    dbInst.execute(sql, [], refreshData);
}

function refreshData(rows, n, rowsAffected) {
    html = "";
    for(var i=0; i<n; i++) {
        console.log(rows[i].timestamp + " / " + rows[i].lat + " / " + rows[i].lon + " / " + rows[i].distance );
        html += "<tr><td width='50%'>"+convertTimeStamp(rows[i].timestamp)+"</td>";
        html += "<td width='20%'>"+rows[i].distance.toFixed(2)+"</td></tr>";
    }
    
    // List traveling nodes:
    $("#listview").html(html);  
    
    // Add a marker on Google Maps
    center = new google.maps.LatLng(rows[n-1].lat, rows[n-1].lon);
    marker = new google.maps.Marker({ 
        position: center, icon: "img/my-dot.png",   
        });
    marker.setMap(theMap); 
    theMap.setCenter(marker.getPosition());        
}

function resetRoute() {
    sql="DELETE FROM route";
    dbInst.execute(sql, []); 
    press_start = true;
    navigator.geolocation.getCurrentPosition(onSuccess, onError);  
    gMap();
}

function getTimeStamp() {
    var today = new Date();
    return Math.floor(today.getTime() / (1000*60)); // minutes
}

function convertTimeStamp(stamp) {
    // stamp : minutes
    var a = new Date(stamp * 60 * 1000); // milliseconds
    return lead_zero(a.getHours(), 2) + ":" + lead_zero(a.getMinutes(), 2);
}

function lead_zero(num, places) {
  var zero = places - num.toString().length + 1;
  return Array(+(zero > 0 && zero)).join("0") + num;
}

function geoDistance(lat1, lon1, lat2, lon2) {
    const R = 6371e3; // metres
    const phee1 = lat1 * Math.PI/180; // φ, λ in radians
    const phee2 = lat2 * Math.PI/180;
    const delta_phee = (lat2-lat1) * Math.PI/180;
    const delta_lambda = (lon2-lon1) * Math.PI/180;

    const a = Math.sin(delta_phee/2) * Math.sin(delta_phee/2) +
              Math.cos(phee1) * Math.cos(phee2) *
              Math.sin(delta_lambda/2) * Math.sin(delta_lambda/2);
    const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));

    const d = R * c; // in metres
    return d;
}

   
